describe 'Given an element', ->

    describe 'when inject is invoked', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget />').inject()

        it 'should return a reference to the jQuery wrapped widget', ->
            expect(widget.is('widget')).toBeTruthy()

    describe 'when no url can be found', ->

        beforeEach ->
            $('<widget />').inject()

        it 'should do nothing', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when a url is provided in the configuration', ->

        beforeEach ->
            $('<widget />').inject
                url: '/configuration'

        it 'should use the configured url', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '/configuration'

    describe 'when a url is found on the element', ->

        beforeEach ->
            $('<widget src="/element" />').inject()

        it 'should use the url from the element', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '/element'

    describe 'when a url is provided in the configuration and on the element', ->

        beforeEach ->
            $('<widget src="/element" />').inject
                url: '/configuration'

        it 'should use the url from the element', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '/element'

    describe 'when the base url is provided on the element', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget base="/base/" src="/element" />').inject()

        it 'should use the base from the element', ->
            expect(widget[0]._base).toEqual '/base/'

    describe 'when the base url is provided on the element and configuration', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget base="/element/" src="/" />').inject
                base: '/config/'

        it 'should use the base from the element', ->
            expect(widget[0]._base).toEqual '/element/'

    describe 'when loading a resource', ->

        beforeEach ->
            $('<widget src="/element" />').inject()

        it 'should GET the resource', ->
            expect($.ajax.getCall(0).args[0].type).toEqual 'get'

    describe 'when using the default configuration', ->

        options = undefined
        element = undefined

        beforeEach ->
            element = $('<widget src="/element" />').inject()
            options = $.ajax.getCall(0).args[0]

        it 'should have the default timeout', ->
            expect(options.timeout).toEqual 4000

        it 'should have the default async behavior', ->
            expect(options.async).toEqual true

        it 'should have the default cache behavior', ->
            expect(options.cache).toEqual false

        it 'should use the jQuery element as the context', ->
            expect(options.context.is('widget')).toEqual true

        it 'should prevent jQuery from inspecting the server response', ->
            expect(options.dataType).toEqual 'text'

        it 'should have the default base', ->
            expect(element[0]._base).toEqual '/'

        it 'should assign the default css class to the widget', ->
            expect(element.hasClass('widget-loading')).toBeTruthy()

    describe 'when the css class is provided in the configuration', ->

        element = undefined

        beforeEach ->
            element = $('<widget src="/" />').inject
                css: { loading: 'custom-class' }

        it 'should assign the configuration-specified css class to the widget', ->
            expect(element.hasClass('custom-class')).toBeTruthy()

    describe 'when attempting to call "inject" on an element more than once', ->

        beforeEach ->
            element = $('<widget src="/element" />').inject().inject()

        it 'should only allow the first call to proceed', ->
            expect($.ajax.calledOnce).toBeTruthy()

    beforeEach ->
        @xhr = sinon.useFakeXMLHttpRequest()
        sinon.spy($, "ajax")

    afterEach ->
        @xhr.restore()
        $.ajax.restore()

describe 'Given unformatted base and/or src urls', ->

    describe 'when location is provided by the current environment', ->

        widget = undefined

        beforeEach ->
            setLocation('domain.dev')
            widget = $('<widget base="http://apps.domain.{0}/" src="/">').inject()
        afterEach -> clearLocation()

        it 'should use the tld from the current environment', ->
            expect(widget[0]._base).toEqual 'http://apps.domain.dev/'

    describe 'when current environment location could not be deteremined', ->

        widget = undefined

        beforeEach ->
            setLocation('single-level-domain')
            widget = $('<widget base="http://apps.domain.{0}/" src="/">').inject()
        afterEach -> clearLocation()

        it 'should use the default ".com" tld', ->
            expect(widget[0]._base).toEqual 'http://apps.domain.com/'

    setLocation = (hostname) ->
        window.widget_location = window.widget_location ||
            hostname: hostname
    clearLocation = ->
        window.widget_location = undefined;

describe 'Given a configuration-provided base', ->

    describe 'when no base is specified and the widget location is an absolute url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="/absolute/">').inject()[0]

        it 'should compute the base as the root path of current location', ->
            expect(widget._base).toEqual '/'

        it 'should load the widget from the url indicated', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '/absolute/'

    describe 'when no base is specified and the widget location is a relative url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="current/../relative/">').inject()[0]

        it 'should NOT compute the base url', ->
            expect(widget._base).not.toBeDefined()

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when no base is specified and the widget location is a schemaless url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="//somedomain.com/item">').inject()[0]

        it 'should compute the base as the authority + root path of the widget location', ->
            expect(widget._base).toEqual '//somedomain.com/'

        it 'should load the widget from the url indicated', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '//somedomain.com/item'

    describe 'when no base is specified and the widget location is a fully qualified url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="https://somedomain.com/item?value=">').inject()[0]

        it 'should compute the base as the scheme + authority + root path of the widget location', ->
            expect(widget._base).toEqual 'https://somedomain.com/'

        it 'should load the widget from the url indicated', ->
            expect($.ajax.getCall(0).args[0].url).toEqual 'https://somedomain.com/item?value='

    describe 'when the configured base is a relative url', ->

        widget = undefined

        beforeEach ->
             widget = $('<widget src="/">').inject(base: './relative')[0]

        it 'should NOT compute the base url', ->
            expect(widget._base).not.toBeDefined()

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is an absolute url and the widget contains an absolute url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="/absolute/">').inject(base: '/base/')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual '/base/'

        it 'should load the widget from the base + absolute url', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '/base/absolute/'

    describe 'when the configured base is an absolute url and the widget contains a relative url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="./relative/">').inject(base: '/base/')[0]

        it 'should NOT compute the base url', ->
            expect(widget._base).not.toBeDefined()

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a absolute url and the widget contains a schemaless url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="//somdomain.com/base/widget.res">').inject(base: '/base/')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual '/base/'

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is an absolute url and the widget contains a fully qualified url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="http://domain.com/base/widget.res">').inject(base: '/base/')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual '/base/'

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a schemaless url and the widget contains an absolute url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="/absolute">').inject(base: '//somedomain.com/app/?query=')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual '//somedomain.com/app/?query='

        it 'should load the widget resource using the configured base and widget location', ->
            expect($.ajax.getCall(0).args[0].url).toEqual '//somedomain.com/app/?query=/absolute'

    describe 'when the configured base is a schemaless url and the widget contains a relative url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="relative/item">').inject(base: '//somedomain.com/app/?query=')[0]

        it 'should NOT compute the base url', ->
            expect(widget._base).not.toBeDefined()

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a schemaless url and the widget contains a schemaless url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="//somdomain.com/app/?query=widget">').inject(base: '//somedomain.com/app/?query=')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual '//somedomain.com/app/?query='

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a schemaless url and the widget contains a fully qualified url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="http://somdomain.com/app/?query=widget">').inject(base: '//somedomain.com/app/?query=')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual '//somedomain.com/app/?query='

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a fully qualified url and the widget contains an absolute url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="/absolute/">').inject(base: 'https://somedomain.com/app/')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual 'https://somedomain.com/app/'

        it 'should load the widget resource using the configureed base and widget location', ->
            expect($.ajax.getCall(0).args[0].url).toEqual 'https://somedomain.com/app/absolute/'

    describe 'when the configured base is a fully qualified url and the widget contains a relative url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="relative/url">').inject(base: 'https://somedomain.com/app/')[0]

        it 'should NOT compute the base url', ->
            expect(widget._base).not.toBeDefined()

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a fully qualified url and the widget contains a schemaless url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="//somdomain.com/widget">').inject(base: 'https://somedomain.com/app/')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual 'https://somedomain.com/app/'

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base is a fully qualified url and the widget contains a fully qualified url', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="http://somdomain.com/app/widget">').inject(base: 'http://somedomain.com/app/')[0]

        it 'should use the configured base', ->
            expect(widget._base).toEqual 'http://somedomain.com/app/'

        it 'should NOT load the widget', ->
            expect($.ajax.called).toBeFalsy()

    describe 'when the configured base does not contain any path information', ->

        widget = undefined

        beforeEach ->
            widget = $('<widget src="/">').inject(base: 'http://somedomain.com')[0]

        it 'should compute the base as the root path of the value provided', ->
            expect(widget._base).toEqual 'http://somedomain.com/'

    beforeEach ->
        @xhr = sinon.useFakeXMLHttpRequest()
        sinon.spy($, "ajax")

    afterEach ->
        @xhr.restore()
        $.ajax.restore()

describe 'Given a widget', ->

    describe 'when making a request to the server', ->

        count = 0

        beforeEach ->
            count = 0
            element = $('<widget src="/element" />').inject
                progress: (value) -> count = value

            respondWith(200)
            server.respond()

        it 'should provide the attempt sequence to the configured "progress" callback', ->
            expect(count).toEqual 1

    describe 'when the server returns "2xx"', ->

        element = null
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                receive: (xhr, responseText) ->
                    result.receive = result.receive || {}
                    result.receive.context = this
                    result.receive.count = (result.receive.count || 0) + 1
                    result.receive.xhr = xhr
                    return result.receive.text = responseText
                append: (html) ->
                    result.append = result.append || {}
                    result.append.context = this
                    result.append.count = (result.append.count || 0) + 1
                    result.append.html = html
                success: (xhr, responseText) ->
                    result.success = result.success || {}
                    result.success.context = this
                    result.success.count = (result.success.count || 0) + 1
                    result.success.xhr = xhr
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.context = this
                    result.complete.count = (result.complete.count || 0) + 1
                    result.complete.xhr = xhr

            respondWith(200)
            server.respond()

        it 'should invoke the configured "receive" callback', ->
            expect(result.receive.count).toEqual 1

        it 'should provide the raw xhr to the configured "receive" callback', ->
            expect(result.receive.xhr).toBeDefined()

        it 'should use the jQuery element as the context for the "recive" callback', ->
            expect(result.receive.context.is('widget')).toEqual true

        it 'should provide the raw response text to the configured "receive" callback', ->
            expect(result.receive.text).toEqual

        it 'should invoke the configured "append" callback', ->
            expect(result.append.count).toEqual 1

        it 'should use the jQuery element as the context for the "append" callback', ->
           expect(result.append.context.is('widget')).toEqual true

        it 'should provide the transformed set of elements to the configured "append" callback', ->
            expect(result.append.html.toLowerCase()).toEqual '<p>hello, world!</p>'

        it 'should invoke the configured "success" callback', ->
            expect(result.success.count).toEqual 1

        it 'should provide the raw xhr to the configured "success" callback', ->
            expect(result.success.xhr).toBeDefined()

        it 'should use the jQuery element as the context for the "success" callback', ->
            expect(result.success.context.is('widget')).toEqual true

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should provide the raw xhr to the configured "complete" callback', ->
            expect(result.complete.xhr).toBeDefined()

        it 'should use the jQuery element as the context for the "complete" callback', ->
            expect(result.complete.context.is('widget')).toEqual true

        it 'should remove the configured "loadingCss" class from the element', ->
            expect(element.hasClass('widget-loading')).toBeFalsy()

        it 'should append the configured success css class to the element', ->
            expect(element.hasClass('widget-loaded')).toBeTruthy()

    describe 'when the server returns "404"', ->

        element = null
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                failure: (xhr) ->
                    result.failure = result.failure || {}
                    result.failure.context = this
                    result.failure.count = (result.failure.count || 0) + 1
                    result.failure.xhr = xhr
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.context = this
                    result.complete.count = (result.complete.count || 0) + 1
                    result.complete.xhr = xhr

            respondWith(404)
            server.respond()

        it 'should invoke the configured "failure" callback', ->
            expect(result.failure.count).toEqual 1

        it 'should use the jQuery element as the context for the "failure" callback', ->
            expect(result.failure.context.is('widget')).toEqual true

        it 'should provide the raw xhr to the configured "failure" callback', ->
            expect(result.failure.xhr).toBeDefined()

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should provide the raw xhr to the configured "complete" callback', ->
            expect(result.complete.xhr).toBeDefined()

        it 'should use the jQuery element as the context for the "complete" callback', ->
            expect(result.complete.context.is('widget')).toEqual true

        it 'should remove the configured css class from the element', ->
            expect(element.hasClass('widget-loading')).toBeFalsy()

    describe 'when the server "500"', ->

        DEFAULT_RETRY_ATTEMPTS = 3

        beforeEach ->
            sinon.spy($, "ajax")

        afterEach ->
            $.ajax.restore()

        beforeEach ->
            element = $('<widget src="/element" />').inject()
            respondWith(500)
            server.respond()

        it 'should retry the request', ->
            expect($.ajax.callCount).toEqual DEFAULT_RETRY_ATTEMPTS

    describe 'when the server does not respond', ->

        DEFAULT_RETRY_ATTEMPTS = 3

        beforeEach ->
            sinon.spy($, "ajax")

        afterEach ->
            $.ajax.restore()

        beforeEach ->
            element = $('<widget src="/element" />').inject
                timeout: 1
            waits(10)

        # This spec works great, but it fails incrementally due to the waits(10) about
        xit 'should retry the request', ->
            expect($.ajax.callCount).toEqual DEFAULT_RETRY_ATTEMPTS

    describe 'when the maximum number of retries has been reached without a "200" response', ->

        element = undefined;
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                failure: (xhr) ->
                    result.failure = result.failure || {}
                    result.failure.context = this
                    result.failure.count = (result.failure.count || 0) + 1
                    result.failure.xhr = xhr
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.context = this
                    result.complete.count = (result.complete.count || 0) + 1
                    result.complete.xhr = xhr

            respondWith(500)
            server.respond()

        it 'should invoke the configured "failure" callback exactly one', ->
            expect(result.failure.count).toEqual 1

        it 'should use the jQuery element as the context for the "failure" callback', ->
            expect(result.failure.context.is('widget')).toEqual true

        it 'should provide the raw xhr to the configured "failure" callback', ->
            expect(result.failure.xhr).toBeDefined()

        it 'should invoke the configured "complete" callback exactly once', ->
            expect(result.complete.count).toEqual 1

        it 'should use the jQuery element as the context for the "complete" callback', ->
            expect(result.complete.context.is('widget')).toEqual true

        it 'should provide the raw xhr to the configured "complete" callback', ->
            expect(result.complete.xhr).toBeDefined()

        it 'should remove the configured "success" css class from the element', ->
            expect(element.hasClass('widget-loading')).toBeFalsy()

        it 'should append the failure css class from the element', ->
            expect(element.hasClass('widget-failed')).toBeTruthy()

    describe 'when the configured "progress" callback throws an exception during a retry', ->

        element = undefined
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                progress: (count) -> if count > 1 then throw 'progress exception'
                exception: (value) ->
                    result.exception = result.complete || {}
                    result.exception.context = this
                    result.exception.count = (result.exception.count || 0) + 1
                    result.exception.value = value
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.count = (result.complete.count || 0) + 1

            respondWith(500)
            server.respond()

        it 'should invoke the configured "exception" callback', ->
            expect(result.exception.count).toEqual 1

        it 'should use the jQuery element as the context for the "exception" callback', ->
            expect(result.exception.context.is('widget')).toBeTruthy()

        it 'should provide the exception to the configured "exception" callback', ->
            expect(result.exception.value).toEqual 'progress exception'

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should append the configured failure css to the element', ->
            expect(element.hasClass('widget-failed')).toBeTruthy()

    describe 'when the configured "receive" callback throws an exception', ->

        element = undefined
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                receive: (xhr) -> throw 'receive exception'
                exception: (value) ->
                    result.exception = result.complete || {}
                    result.exception.context = this
                    result.exception.count = (result.exception.count || 0) + 1
                    result.exception.value = value
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.count = (result.complete.count || 0) + 1

            respondWith(200)
            server.respond()

        it 'should invoke the configured "exception" callback', ->
            expect(result.exception.count).toEqual 1

        it 'should use the jQuery element as the context for the "exception" callback', ->
            expect(result.exception.context.is('widget')).toBeTruthy()

        it 'should provide the exception to the configured "exception" callback', ->
            expect(result.exception.value).toEqual 'receive exception'

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should append the configured failure css to the element', ->
            expect(element.hasClass('widget-failed')).toBeTruthy()

    describe 'when the configured "append" callback throws an exception', ->

        element = undefined
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                append: (xhr) -> throw 'append exception'
                exception: (value) ->
                    result.exception = result.complete || {}
                    result.exception.context = this
                    result.exception.count = (result.exception.count || 0) + 1
                    result.exception.value = value
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.count = (result.complete.count || 0) + 1

            respondWith(200)
            server.respond()

        it 'should invoke the configured "exception" callback', ->
            expect(result.exception.count).toEqual 1

        it 'should use the jQuery element as the context for the "exception" callback', ->
            expect(result.exception.context.is('widget')).toBeTruthy()

        it 'should provide the exception to the configured "exception" callback', ->
            expect(result.exception.value).toEqual 'append exception'

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should append the configured failure css to the element', ->
            expect(element.hasClass('widget-failed')).toBeTruthy()

    describe 'when the configured "success" callback throws an exception', ->

        element = undefined
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                success: (xhr) -> throw 'success exception'
                exception: (value) ->
                    result.exception = result.complete || {}
                    result.exception.context = this
                    result.exception.count = (result.exception.count || 0) + 1
                    result.exception.value = value
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.count = (result.complete.count || 0) + 1

            respondWith(200)
            server.respond()

        it 'should invoke the configured "exception" callback', ->
            expect(result.exception.count).toEqual 1

        it 'should use the jQuery element as the context for the "exception" callback', ->
            expect(result.exception.context.is('widget')).toBeTruthy()

        it 'should provide the exception to the configured "exception" callback', ->
            expect(result.exception.value).toEqual 'success exception'

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should append the configured failure css to the element', ->
            expect(element.hasClass('widget-failed')).toBeTruthy()

    describe 'when the configured "failure" callback throws an exception', ->

        element = undefined
        result = {}

        beforeEach ->
            result = {}

            element = $('<widget src="/element" />').inject
                failure: (xhr) -> throw 'failure exception'
                exception: (value) ->
                    result.exception = result.complete || {}
                    result.exception.context = this
                    result.exception.count = (result.exception.count || 0) + 1
                    result.exception.value = value
                complete: (xhr) ->
                    result.complete = result.complete || {}
                    result.complete.count = (result.complete.count || 0) + 1

            respondWith(404)
            server.respond()

        it 'should invoke the configured "exception" callback', ->
            expect(result.exception.count).toEqual 1

        it 'should use the jQuery element as the context for the "exception" callback', ->
            expect(result.exception.context.is('widget')).toBeTruthy()

        it 'should provide the exception to the configured "exception" callback', ->
            expect(result.exception.value).toEqual 'failure exception'

        it 'should invoke the configured "complete" callback', ->
            expect(result.complete.count).toEqual 1

        it 'should append the configured failure css to the element', ->
            expect(element.hasClass('widget-failed')).toBeTruthy()

    server = undefined

    beforeEach ->
        server = sinon.fakeServer.create()

    afterEach ->
        server.restore()

    respondWith = (statusCode) ->
        server.respondWith([statusCode, { 'Context-Type': 'text/html' }, '<p>Hello, World!</p>' ])

describe 'Given content to be injectd', ->

    describe 'when the content is received', ->

        beforeEach ->
            execute('Hello, World!')

        it 'should add the content body of the widget', ->
            expect(appended).toEqual 'Hello, World!'

    describe 'when adding an element with a relative src', ->

        beforeEach ->
            execute('<custom src="../relative/some.resource"></custom>')

        it 'should rebase the element with the src location relative to the widget location and base url', ->
            expect(appended).toEqual('<custom src="../relative/some.resource"></custom>')

    describe 'when adding an element with an absolute src', ->

        beforeEach ->
            execute('<custom src="/absolute/some.resource"></custom>')

        it 'should rebase the element while considering the base url', ->
            expect(appended).toEqual('<custom src="/base/absolute/some.resource"></custom>')

    describe 'when adding an element with a schemaless src', ->

        beforeEach ->
            execute('<custom src="//www.google.com/absolute/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element with a fully qualified src', ->

        beforeEach ->
            execute('<custom src="http://www.google.com/absolute/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element containing an absolute src along with a rebase="none" attribute', ->

        beforeEach ->
            execute('<custom src="/" rebase="none"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding element with a relative href', ->

        beforeEach ->
            execute('<custom href="../../relative/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element with an absolute href', ->

        beforeEach ->
            execute('<custom href="/absolute/some.resource"></custom>')

        it 'should rebase the element while considering the base url', ->
            expect(appended).toEqual('<custom href="/base/absolute/some.resource"></custom>')

    describe 'when adding an element with a schemaless href', ->

        beforeEach ->
            execute('<custom href="//www.google.com/absolute/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element with a fully qualified href', ->

        beforeEach ->
            execute('<custom href="http://www.google.com/absolute/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element containing an absolute href along with a rebase="none" attribute', ->

        beforeEach ->
            execute('<custom href="/" rebase="none"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an elementing containing where the href value starts with "javascript:"', ->

        beforeEach ->
            execute('<custom href="javascript:doSomething();"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an elementing containing where the href value starts with a hash tag', ->

        beforeEach ->
            execute('<custom href="#tag_on_page"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding element with a relative action', ->

        beforeEach ->
            execute('<custom action="../../../relative/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element with an absolute action', ->

        beforeEach ->
            execute('<custom action="/absolute/some.resource"></custom>')

        it 'should rebase the element according to the the base url', ->
            expect(appended).toEqual('<custom action="/base/absolute/some.resource"></custom>')

    describe 'when adding an element with a schemaless action', ->

        beforeEach ->
            execute('<custom action="//www.google.com/absolute/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding an element with a fully qualified action', ->

        beforeEach ->
            execute('<custom action="http://www.google.com/absolute/some.resource"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when the element\'s src|href|action is empty', ->

        beforeEach ->
            execute('<custom action=""></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual('<custom action=""></custom>')

    describe 'when the element\'s src|href|action attribute starts with the specialized "widget://base/" substitution string', ->

        beforeEach ->
            execute('<custom action="widget://base/"></custom>')

        it 'should rebase the element exactly once', ->
            expect(appended).toEqual('<custom action="/base/"></custom>')

    describe 'when the content contains a javascript-embedded *empty* url', ->

        content = '
            <script type="text/javascript">
                $.ajax({
                    url: "",
                    type: "get"
                });
            </script>
        '

        beforeEach ->
            execute(content)

        it 'should NOT rebase the url', ->
            expect(appended.indexOf('url: "",') > 0).toBeTruthy()

    describe 'when the content contains a javascript-embedded relative url', ->

        content = '
            <script type="text/javascript">
                $.ajax({
                    url: "relative/some-page",
                    type: "get"
                });
            </script>'

        beforeEach ->
            execute(content)

        it 'should NOT rebase the url', ->
            expect(appended.indexOf('url: "relative/some-page",') > 0).toBeTruthy()

    describe 'when the content contains a javascript-embedded absolute url', ->

        content = '
            <script type="text/javascript">
                $.ajax({
                    url: "/absolute/",
                    type: "get"
                });
            </script>'

        beforeEach ->
            execute(content)

        it 'should rebase the url according to the the base url', ->
            expect(appended.indexOf('url: "/base/absolute/",') > 0).toBeTruthy()

    describe 'when the content contains a javascript-embedded schemaless url', ->

        content = '
            <script type="text/javascript">
                $.ajax({
                    url: "//domain.com/whatever",
                    type: "get"
                });
            </script>'

        beforeEach ->
            execute(content)

        it 'should NOT rebase the url', ->
            expect(appended.indexOf('url: "//domain.com/whatever",') > 0).toBeTruthy()

    describe 'when the content contains a javascript-embedded fully qualified url', ->

        content = '
            <script type="text/javascript">
                $.ajax({
                    url: "https://domain.com/whatever",
                    type: "get"
                });
            </script>'

        beforeEach ->
            execute(content)

        it 'should NOT rebase the url', ->
            expect(appended.indexOf('url: "https://domain.com/whatever",') > 0).toBeTruthy()

    describe 'when adding an element containing an absolute action along with a rebase="none" attribute', ->

        beforeEach ->
            execute('<custom action="/" rebase="none"></custom>')

        it 'should NOT rebase the element', ->
            expect(appended).toEqual(original)

    describe 'when adding content containing the specialized "widget://base/" substitution string', ->

        beforeEach ->
            execute('base is defined as widget://base/')

        it 'should replace the variable with the configured base', ->
            expect(appended).toEqual('base is defined as /base/')

    describe 'when the element\'s src|href|action attribute contains the specialized "widget://base/" substitution string', ->

        beforeEach ->
            execute('<custom action="/urlwidget://base/"></custom>')

        it 'should rebase the element exactly once', ->
            expect(appended).toEqual('<custom action="/url/base/"></custom>')

    server = undefined
    original = undefined
    appended = undefined

    beforeEach ->
        server = sinon.fakeServer.create()
        original = undefined
        appended = undefined

    afterEach ->
        server.restore()

    execute = (value, url, base) ->
        original = value
        server.respondWith([200, { 'Context-Type': 'text/html' }, value || '' ])
        $('<widget src="' + ( url || '/widget/path/widget.htm' ) + '" />').inject
            base: (base || '/base/')
            append: (raw) -> appended = raw
        server.respond()